
-- Based on Settings.lua by Pengoros

-- Default Settings
DEFAULT_SETTINGS =
{
	lang = "FRENCH";
	pluginName="SuperChrono";
	msgColor = "FF0000";
	debug = false;
	log =
	{
		x = 100;
		y = 100;
		visible = true;
	};
	chrono =
	{
		x = 700;
		y = 100;
		visible = true;
	};
	configurations =
	{ 
	};
};



Settings = class();

-- encode numbers as strings if necessary for the client to save properly
function Settings:EncodeNumbers(object)
    local lookup_table = {};
    local function _encode(object)
        if (type(object) == "number") then
            return ("<num>%f</num>"):format(object);
        elseif (type(object) ~= "table") then
            return object;
        elseif (lookup_table[object]) then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
            new_table[_encode(index)] = _encode(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _encode(object);
end

-- decode numbers encoded as strings
function Settings:DecodeNumbers(object)
    local lookup_table = {};
    local function _decode(object)
        if (type(object) == "number") then
        	return object;
        elseif (type(object) == "string") then
            local match = object:match("<num>(\-?%d+.%d+)</num>");
            if (match ~= nil) then
                return tonumber(match);
            else
                return object;
            end
        elseif (type(object) ~= "table") then
            return object;
        elseif lookup_table[object] then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
            new_table[_decode(index)] = _decode(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _decode(object);
end

function Settings:Constructor()

	function _gss()
		return Turbine.PluginData.Load(Turbine.DataScope.Account, "KikiSuperChrono");
	end
	local isOk, saved = pcall(_gss);

	if not isOk or type(saved) ~= "table" then
		if saved == nil then
			Turbine.Shell.WriteLine("SuperChrono: Using default settings");
		else
			Turbine.Shell.WriteLine("SuperChrono: Error loading Settings, using default settings");
		end
		saved = deepcopy(DEFAULT_SETTINGS);
	end
    saved = self:DecodeNumbers(saved);

 	for i,v in pairs(saved) do
 		self[i] = v;
 	end


end

function Settings:_Save()
    local object = Settings:EncodeNumbers(self);
    Turbine.PluginData.Save(Turbine.DataScope.Account, "KikiSuperChrono", object);
end

-- class level stuff
function Settings.Load()
    Settings.instance = Settings();
    return Settings.instance;
end

function Settings.Save()
	Settings.instance:_Save();
end



